@@ -4,7 +4,7 @@ import ee
 import ee.data
 import pandas as pd
 import math
-from . import date
+from . import date, ee_list
 from . import image as image_module
 from . import collection as eecollection
 from ..utils import castImage
@@ -145,42 +145,57 @@ def enumerateSimple(collection, name='ENUM'):
     return ee.ImageCollection(fc.copyProperties(source=collection))
 
 
-def fillWithLast(collection, reverse=False, date_property='system:time_start'):
+def fillWithLast(collection, reverse=False, proxy=-999):
     """ Fill each masked pixels with the last available not masked pixel. If reverse, it goes backwards.
     Images must contain a valid date (system:time_start property by default) """
+    axis = 0
+    def shift(array):
+        if reverse:
+            rigth = array.arraySlice(axis, 1)
+            last = array.arraySlice(axis, -1)
+            return rigth.arrayCat(last, axis)
+        else:
+            left = array.arraySlice(axis, 0, -1)
+            first = array.arraySlice(axis, 0, 1)
+            return first.arrayCat(left, axis)
 
-    def makereverse(col):
-        dates = ee.List(col.aggregate_array(date_property)).reverse()
-        col = ee.ImageCollection.fromImages(ee.List(dates.map(
-            lambda d: ee.Image(
-                col.filter(ee.Filter.eq(date_property, d)).first()))))
-        return col
+    def move(array):
+        shifted = shift(array)
+        masked = array.neq(proxy)
+        maskednot = array.eq(proxy)
+        t1 = array.multiply(masked)
+        t2 = shifted.multiply(maskednot)
+        final = t1.add(t2)
+        return final
 
-    if reverse:
-        collection = makereverse(collection)
+    def fill(array, size):
+        size = ee.Number(size)
+        indices = ee.List.sequence(0, size.subtract(1))
+        def wrap(i, a):
+            a = ee.Image(a)
+            return move(a)
+        return ee.Image(indices.iterate(wrap, array))
 
-    def overcol(i, collect):
-        i = ee.Image(i)
-        collect = ee.List(collect)
+    collection = collection.map(
+        lambda i: image_module.emptyBackground(i, proxy).copyProperties(
+            source=i, properties=i.propertyNames()))
+    bands = ee.Image(collection.first()).bandNames()
+    size = collection.size()
+    array = collection.toArray()
+    fill_array = fill(array, size)
 
-        def true():
-            last = ee.Image(collect.get(-1))
-            blended = i.unmask().where(i.mask().Not(), last)
-            return collect.add(blended)
+    props = aggregate_array_all(collection)
+    indices = ee.List.sequence(0, size.subtract(1))
 
-        def false():
-            return collect.add(i)
+    def wrap(index):
+        index = ee.Number(index).toInt()
+        sliced = fill_array.arraySlice(axis, index, index.add(1))
+        im = sliced.arrayProject([1]).arrayFlatten([bands])
+        prop = ee.Dictionary(props.get(index))
+        im = ee.Image(im.setMulti(prop))
+        return im.updateMask(im.neq(proxy))
 
-        condition = collect.size()
-        return ee.List(ee.Algorithms.If(condition, true(), false()))
-
-    result = ee.ImageCollection.fromImages(
-        ee.List(collection.iterate(overcol, ee.List([]))))
-
-    if reverse:
-        result = makereverse(result)
-
-    return result
+    return ee.ImageCollection.fromImages(indices.map(wrap))
 
 
 def mergeGeometries(collection):
@@ -391,7 +406,7 @@ def reduceDayIntervals(collection, reducer, interval=30, reverse=False,
 
 def getValues(collection, geometry, scale=None, reducer=None,
               id='system:index', properties=None, side='server',
-              maxPixels=1e9):
+              maxPixels=1e7, bestEffort=False, tileScale=1):
     """ Return all values of all bands of an image collection in the
         specified geometry
 
@@ -424,8 +439,10 @@ def getValues(collection, geometry, scale=None, reducer=None,
 
     def listval(img, it):
         theid = ee.Algorithms.String(img.get(id))
-        values = img.reduceRegion(reducer, geometry, scale,
-                                  maxPixels=maxPixels)
+        values = img.reduceRegion(
+            reducer, geometry, scale, maxPixels=maxPixels,
+            bestEffort=bestEffort, tileScale=tileScale
+        )
         values = ee.Dictionary(values)
         img_props = img.propertyNames()
 
@@ -716,11 +733,14 @@ def linearInterpolation(collection, date_property='system:time_start'):
     # get the mask for the final result
     finalmask = allMasked(collection)
 
+    if date_property != 'system:time_start':
+        collection = collection.sort(date_property)
+
     # add time bands
     collection = _addTime(collection)
 
-    filled = fillWithLast(collection, False, date_property=date_property)
-    filled_back = fillWithLast(collection, True, date_property=date_property)
+    filled = fillWithLast(collection, False)
+    filled_back = fillWithLast(collection, True)
 
     condition = ee.Filter.equals(leftField='system:index',
                                  rightField='system:index')
@@ -1036,3 +1056,86 @@ def area_under_curve(collection, band, x_property=None, name='area_under'):
     final = ee.List(collection.iterate(cummulative, ee.List([])))
     final_ic = ee.ImageCollection.fromImages(final).select(name)
     return ee.Image(final_ic.reduce(ee.Reducer.sum()))
+
+
+def moving_average(collection, back=5, reducer=None,
+                   use_original=True):
+    """ Compute the moving average over a time series
+
+    :param back: number of images back to use for computing the stats
+    :type back: int
+    :param reducer: the reducer to apply. Default is ee.Reducer.mean()
+    :type reducer: ee.Reducer
+    :param use_original: if True, computes the stats over the last original
+        values, otherwise, computes the stats over the last computed values
+    :type use_original: bool
+    """
+    if reducer is None:
+        reducer = ee.Reducer.mean()
+
+    def wrap(i, d):
+        d = ee.Dictionary(d)
+        i = ee.Image(i)
+        original = ee.List(d.get('original'))
+        stats = ee.List(d.get('stats'))
+
+        def true(im, di):
+            original_true = ee.List(di.get('original'))
+            stats_true = ee.List(di.get('stats'))
+            original_true = original_true.add(im)
+            tocompute = original_true if use_original else stats_true.add(im)
+            tempcol = ee.ImageCollection.fromImages(tocompute.slice(back * -1))
+            stats = tempcol.reduce(reducer)
+            stats = stats.rename(im.bandNames())
+            stats = ee.Image(stats.copyProperties(im, properties=im.propertyNames()))
+            return ee.Dictionary({
+                'original': original_true,
+                'stats': stats_true.add(stats)
+            })
+
+        def false(im, di):
+            original2 = ee.List(di.get('original'))
+            stats2 = ee.List(di.get('stats'))
+            condition2 = original2.size().gt(0)
+
+            def true2(ima, dic):
+                original_true2 = ee.List(dic.get('original'))
+                original_true2 = original_true2.add(ima)
+                stats_true2 = ee.List(dic.get('stats'))
+                tocompute = original_true2 if use_original else stats_true2.add(ima)
+                tempcol2 = ee.ImageCollection.fromImages(tocompute)
+                stats2 = tempcol2.reduce(reducer)
+                stats2 = stats2.rename(ima.bandNames())
+                stats2 = ee.Image(stats2.copyProperties(ima, properties=ima.propertyNames()))
+                return ee.Dictionary({
+                    'original': original_true2,
+                    'stats': stats_true2.add(stats2)
+                })
+
+            def false2(ima, dic):
+                # first element
+                original_false2 = ee.List(dic.get('original'))
+                stats_false2 = ee.List(dic.get('stats'))
+                return ee.Dictionary({
+                    'original': original_false2.add(ima),
+                    'stats': stats_false2.add(ima)
+                })
+
+            return ee.Dictionary(
+                ee.Algorithms.If(condition2, true2(im, di), false2(im, di)))
+
+        condition = original.size().gte(back)
+        return ee.Dictionary(
+            ee.Algorithms.If(condition, true(i, d), false(i, d)))
+
+    final = ee.Dictionary(
+        collection.iterate(wrap, ee.Dictionary({'original': [], 'stats': []})))
+    return ee.ImageCollection.fromImages(ee.List(final.get('stats')))
+
+
+def aggregate_array_all(collection):
+    """ Aggregate array in all images and return a list of dicts """
+    props = collection.first().propertyNames()
+    allprops = props.map(lambda p: collection.aggregate_array(p))
+    transposed = ee_list.transpose(allprops)
+    return transposed.map(lambda ps: ee.Dictionary.fromLists(props, ps))
